#!/bin/bash

# 定义日志文件路径和锁文件路径
LOGFILE="/var/log/pci_sound_card_hotplug.log"
LOCKFILE="/tmp/pci_sound_card_hotplug.lock"


# 函数：记录日志
log_message() {
    echo "$(date '+%Y-%m-%d %H:%M:%S') - $1" >> "$LOGFILE"
}

# 检查锁文件是否存在
if [ -f "$LOCKFILE" ]; then
    log_message "Lock file exists. Another instance of the script is running or has already run. Exiting."
    exit 1
fi

# 创建锁文件
touch "$LOCKFILE"
log_message "Created lock file: $LOCKFILE."

# 固定的PCI路径
PCI_PATH="0000:00:14.0"

# 主逻辑开始
log_message "Starting PCI sound card hotplug action with fixed path: /sys/bus/pci/devices/$PCI_PATH."

# 检查路径是否存在
if [ ! -f "/sys/bus/pci/devices/$PCI_PATH/remove" ]; then
   log_message "Error: The remove file does not exist at /sys/bus/pci/devices/$PCI_PATH/remove."
   rm -f "$LOCKFILE"
       exit 1
fi

# 卸载声卡
log_message "Unloading sound card..."
echo 1 | sudo tee "/sys/bus/pci/devices/$PCI_PATH/remove" > /dev/null || log_message "Failed to remove PCI device $PCI_PATH."

# 等待一段时间确保设备完全断电
sleep 2

# 重新扫描PCI总线以加载声卡
log_message "Rescanning PCI bus..."
echo 1 | sudo tee /sys/bus/pci/rescan > /dev/null || log_message "Failed to rescan PCI bus."

# 刷新ALSA配置，确保新的音频设备被正确识别
alsactl restore || log_message "Failed to restore ALSA settings."

log_message "PCI sound card hotplug action completed."

